<?
require_once LIB."/users.php";

/* hurrah for declarative programming -- explicit is better than implicit! */

$pollingMethods = array(
    'chooseone' =>  array(
        'name' => 'chooseone',
        'longname' => 'Vote for One Proposition',
        'numprops' => array('2','3+'),
        'drules' => array('plurality', 'consen','consenminus1',
                          'threefourth','twothird','majority')),
    'updown' =>  array(
        'name' => 'updown',
        'longname' => 'Approve/Disapprove',
        'numprops' => array('1','2','3+'),
        'drules' => array('majority','consen','consenminus1',
                          'threefourth','twothird')),
    'allocatepts' =>  array(
        'name' => 'allocatepts',
        'longname' => 'Allocate 100 points',
        'numprops' => array('2','3+'),
        'drules' => array('mostpts','maximin','avgaboveth')),
    'rateindiv' =>  array(
        'name' => 'rateindiv',
        'longname' => 'Rate individually (0-10)',
        'numprops' => array('2','3+'),
        'drules' => array('mostpts','maximin','avgaboveth')),
    'rankall' =>  array(
        'name' => 'rankall',
        'longname' => 'Rank all',
        'numprops' => array('3+'),
        'drules' => array('borda','condorcet','hare','tworunoff','plurality')),
    'ranksome' =>  array(
        'name' => 'ranksome',
        'longname' => 'Rank some',
        'numprops' => array('3+'),
        'drules' => array('borda','hare','tworunoff','plurality'))
);

$drules    = array(
    'plurality' => array(
        'longname' => 'Plurality (most votes wins)'),
    'majority' => array(
        'longname' => 'Majority'),
    'twothird' => array(
        'longname' => 'Two-thirds'),
    'threefourth' => array(
        'longname' => 'Three-fourths'),
    'consenminus1' => array(
        'longname' => 'Consensus except one person'),
    'consen' => array(
        'longname' => 'Consensus (100%)'),
    'mostpts' => array(
        'longname' => 'Most points'),
    'maximin' => array(
        'longname' => 'Maximin'),
    'avgaboveth' => array(
        'longname' => 'Average above a threshold'),
    'borda' => array(
        'longname' => 'Borda count'),
    'condorcet' => array(
        'longname' => 'Condorcet'),
    'hare' => array(
        'longname' => 'Instant Runoff (Hare)'),
    'tworunoff' => array(
        'longname' => 'Two option runoff')
);


/* ------------------------------------------------------------------------- */
/* ----------- INSERT D-RULE and METHODS IMPLEMENTATION CODE HERE!!! ------- */

$implementedDRules = array(
'chooseone_plurality'=>true,
'chooseone_majorty'=>true,
'chooseone_consen'=>true,
'chooseone_consenminus1'=>true,
'chooseone_twothird'=>true,
'chooseone_threefourth'=>true,
'updown_majority'=>true,
'updown_consen'=>true,
'updown_consenminus1'=>true,
'updown_twothird'=>true,
'updown_threefourth'=>true
);
$implementedMethods = array('chooseone'=>true, 'updown'=>true);

/* Each DRule has teh following interface:
 *  contructor($params) -- $params is a nested named array or whatever.
 *  evaluate($poll) -- give us a result and hopefully a winner of the vote.
 *   return an array of winning propositions in format (propid => Prop object).
 *  viewForBallot($poll) -- echos for inside the <form> on the ballot page.
 *
 * Each Tally needs to accept tallyNew Vote, and do a bunch of other things
 * less related to being tallies than they are as code-holders for their respective
 * methods.
 *
 * also, their DRules better know how to use em.
 *
 * UserVotes are wrappers over the userpolls table.. the relationship
 * between one user and one poll.  Typed by poll method; thus heavily interdependent
 * with their repsective Tally type.
 */

/* Tallies and UserVotes. */


/* ------------------------------------------------------------------------
 * Method:  chooseone
 *  makeDefaultVoteData, Tally, UserVote.
 */

function makeDefaultVoteData_chooseone($propids)
{
    return Array('prop_id' => 0);
}

class Tally_chooseone {
    function Tally_chooseone(&$params, $prop_ids) {
        $this->tallies = array();
        foreach($prop_ids as $prop_id) {
            $this->tallies[$prop_id] = 0;
        }
    }
    function tallyNewVote(&$oldVote, &$newVote) {
        $old_prop_id = $oldVote->vote_data['prop_id'];
        $new_prop_id = $newVote->vote_data['prop_id'];

        //php type conversion rules are confusing
        if ($old_prop_id != 0 && $old_prop_id != '0') {
            $this->tallies[$old_prop_id] -= 1;
        }
        $this->tallies[$new_prop_id] += 1;
    }
    function getCommentSubj(&$oldVote, &$newVote, $poll) {
        $subj = $oldVote->hasVoted() ? "Comment, vote change to" : "Vote comment for";
        $npi = $newVote->vote_data['prop_id'];
        $subj .= ": \"".addslashes($poll->props[$npi]->text)."\"";
        return $subj;
    }
    function viewAsSummary(&$poll) {
        echo "<table border=0 class=poll_choice_summary>";

        echo "<tr style=\"text-align:left;text-decoration:underline;\">";
        echo "<th>Choices";
        echo "<th>"; 
        echo $poll->isOpen() ? "Votes so far" : "Total votes";
        echo "</tr>";

        foreach($this->tallies as $prop_id => $proptally) {
            if(! $prop_id) continue; //this is a hack for corrupted tallies
                //still haven't found the *cause* of these corrupted tallies yet
            echo "<tr>";
            $text = $poll->props[$prop_id]->text;
            echo "<td>";
            if(! $poll->is_straw) {
                echo "<a href=\"javascript:frameset.popup_fullpropview_window($prop_id)\">";
                echo htmlspecialchars($text);
                echo "</a>";
            } else {
                echo htmlspecialchars($text);
            }
            echo "<td style=\"text-align:center;\">".$proptally;
        }
        echo "</table>";
    }
    function viewChoicesWithDetails(&$poll) {
        echo "<table border=0 class=poll_choice_summary>";

        echo "<tr style=\"text-align:left;text-decoration:underline;\">";
        echo "<th>Choices";
        echo "<th>"; 
        echo $poll->isOpen() ? "Votes so far" : "Total votes";
        echo "</tr>";

        foreach($this->tallies as $prop_id => $proptally) {
            echo "<tr>";
            $text = $poll->props[$prop_id]->text;
            echo "<td>".htmlspecialchars($text);
            echo "<td style=\"text-align:center;\">".$proptally;
            //---- details.
            echo "<tr>";
            echo "<td colspan=3 width=\"100%\">";
            echo "<div style=\"position:relative; left:20px; right:20px;\">";
            echo $poll->props[$prop_id]->makeDetailedView();
            echo "</div>";
            echo "<HR>";
        }
        echo "</table>";
    }

    function getImplAction($actions) {
        $impls = array_filter($actions, create_function('$a','return $a->type == "ImplementAction";'));
        return $impls ? array_pop($impls) : false;
    }

    function getResultsReportText(&$poll) {
        $winners = $poll->drule->evaluate($poll);
        if(count($winners) == 1) {
            $prop = array_pop($winners);
            $s = "The winner was the proposition \"".$prop->text."\".";
            $s .= "\n";
            $n = $this->tallies[$prop->prop_id];
            $s .= "  with $n votes.\n";
            $impl = $this->getImplAction($prop->actions);
            if($impl && $impl->action_details['text']) {
                $s .= "\nImplementation plan:\n\n".$impl->action_details['text']."\n";
            }
        } else if(count($winners) > 1) {
            $s  = "There were ".count($winners)." winning propositions:\n\n";
            foreach($winners as $prop) {
                $s .= "* ".$prop->text."\n";
                $n = $this->tallies[$prop->prop_id];
                $s .= "  with $n votes.\n";
                $impl = $this->getImplAction($prop->actions);
                if($impl && $impl->action_details['text']) {
                    $s .= "Implementation plan:\n".$impl->action_details['text']."\n";
                }
                $s .= "\n";
            }
        } else if(count($winners) == 0) {
            $s = "No propositions were winners.\n";
        }
        return $s;
    }
    
    /* @access private . */
    function sortIntoBins(&$uservotes) {
        $bins = array();
        foreach($uservotes as $uv) {
            if($uv->hasVoted()) {
                $pi = $uv->vote_data['prop_id'];
            } else {
                $pi = 0;
            }
            if(!isset($bins[$pi])) {
                $bins[$pi] = array();
            }
            array_push($bins[$pi], $uv);
        }
        return $bins;
    }

    function viewAsBreakdown(&$poll) {
        $userVotesByPropId =& $this->sortIntoBins($poll->userVotes);
        echo "<table class=poll_breakdown><tr><td>";
        echo "<span class=poll_breakdown_title>Member choices/comments:</span>";
        echo "<table class=poll_breakdown_entries>";
        foreach($userVotesByPropId as $prop_id => $userVotes) {
            if($prop_id == 0) {continue;}
            echo "<TR><TH colspan=2>";
            echo $poll->props[$prop_id]->text;
            foreach($userVotes as $uv) {
                echo "<TR>";
                echo "<TD>".make_linked_username($uv->username);
                echo "<TD style=\"font-size:80%\">";
                echo strftime("%b-%d,&nbsp;%R", strtotime($uv->when_voted));
                if($uv->msgtext) {
                    echo "<TR><TD>&nbsp;";
                    echo "<TD colspan=2 style=\"font-size:80%;left:5px;\">";
                    echo "<I>".htmlspecialchars($uv->msgtext)."</I>";
                }
            }
            echo "<TR><TD>&nbsp;<TD>&nbsp;</TR>";
        }
        if(isset($userVotesByPropId[0])) {
            echo "<TR><TH colspan=2>";
            echo $poll->isOpen() ? "Have not voted:" : "Did not vote:";
            
            $usernames = array();
            foreach($userVotesByPropId[0] as $uv) {
                $usernames[] = make_linked_username($uv->username);
            }
            echo "<tr><td colspan=2>";
            echo implode(", ", $usernames);
            echo "</tr>";
        }
        echo "</table>";
        echo "</table>";
    }

    function viewAsBallotOptions(&$poll, &$uservote) {
        echo "<table class=poll_choice_summary><TR><TD>";
        foreach($poll->props as $prop_id => $prop) {
            echo "<input type=radio name=voted_prop_id value=$prop->prop_id ";
            if ($prop->prop_id == $uservote->vote_data['prop_id']) {
                echo " checked=checked ";
            }
            echo ">";
            if(! $poll->is_straw) {
                echo "<a href=\"javascript:frameset.popup_fullpropview_window($prop_id)\">";
                echo "<b>".htmlspecialchars($prop->text)."</b>";
                echo "</a>";

                echo "<tr><td>";
                echo "<i>Implementation details:</i> ";
                echo $prop->getImplementationDetails();
                echo "<BR>";
            } else {
                echo "<b>".htmlspecialchars($prop->text)."</b>";
            }
            echo "<BR>";
        }
        echo "</table>";
    }
}

class UserVote_chooseone extends UserVote {
    function viewCurrentVote(&$poll) {
        assert($this->hasVoted());

        echo "Your current vote is for &quot;";
        $text = $poll->props[$this->vote_data['prop_id']]->text;
        echo htmlspecialchars($text);
        echo "&quot;";
        echo "<BR>";
        echo "<span style=\"font-size:30%;\">&nbsp;</span><BR>";
        echo "<a class=vote_button href=\"";
        echo $_SERVER['REQUEST_URI'] . "&as_ballot=1";
        echo "\">Change your vote</a>";
        echo "<BR>";
        echo "<span style=\"font-size:30%;\">&nbsp;</span><BR>";
    }

    function processVoteSubmission($db, &$post) {
        $old_prop_id = $this->vote_data['prop_id'];
        $new_prop_id = $post['voted_prop_id'];

        $this->vote_data['prop_id'] = $new_prop_id;
        $ser = addslashes(serialize($this->vote_data));
        $q = "UPDATE userpolls SET vote_data = '$ser',
                                   when_voted = NOW()
              WHERE username = '".$this->username."' AND
                    poll_id  =  ".$this->poll_id;
        $db->query($q);
    }
}

/* ------------------ end method chooseone ---------------- */

/* ----- Method: Approve/Disapprove Each Prop  ('updown') ---- */

//constants.  I am NOT using $var globals due to a crazy php bug
//just found, where functions called via call_user_func() or eval() do not
//correctly handle global variables accessible to them in certain situations.
//php is, well, overstretched
define("updown_YES", "-1");
define("updown_NO", "0");
define("updown_NOVOTEYET", "1");


function updown_choiceAsEnglish($choice)
{
//     echo "asEnglish: ".$choice."<br>\n";
    if($choice == updown_YES)
        return "Approve";
    else if($choice == updown_NO)
        return "Disapprove";
    else if($choice == updown_NOVOTEYET)
        return "Has Not Voted";
}


function updown_choiceAsHtml($choice)
{
    global $CONF;
    $s = "";
    if($choice == updown_YES) {
        $s .= "<img src=".$CONF['IMAGES']."/yes-check.gif>";
    } else if($choice == updown_NO) {
        $s .= "<img src=".$CONF['IMAGES']."/no-check.gif>";
    }
    $s .= updown_choiceAsEnglish($choice);
    return $s;
}

class Tally_updown {

    function Tally_updown(&$params, $prop_ids) {
        //don't count abstentions.  1) can deduce from userlist. 2) userlist
        //needed to make them, but doesnt get passed to Tally constructor (as
        //of yet)
        $this->tallies = Array();
        foreach($prop_ids as $prop_id) {
            $newTallyForProp = array(updown_YES => 0, updown_NO => 0);
            $this->tallies[$prop_id] = $newTallyForProp;
        }
    }

    /* @params both are UserVotes */
    function tallyNewVote(&$oldVote, &$newVote) {
        assert(count($oldVote->vote_data) == count($newVote->vote_data));
        foreach($oldVote->vote_data as $prop_id => $v) {
            $oldv = $oldVote->vote_data[$prop_id];
            $newv = $newVote->vote_data[$prop_id];
            if($oldv != updown_NOVOTEYET) {
                $this->tallies[$prop_id][$oldv]--;
            }
            if($newv != updown_NOVOTEYET) {
                $this->tallies[$prop_id][$newv]++;
            }
        }
    }

    function getCommentSubj(&$oldVote, &$newVote, $poll) {
        $subj = $oldVote->hasVoted() ? "Comment, vote change" : "Vote comment";
        return $subj;
    }

    function viewAsSummary(&$poll) {
        echo "<table border=0 class=poll_choice_summary>";
        echo "<tr style=\"text-align:left;text-decoration:underline;\">";
        echo "<th>Propositions";
        echo "<th><i>Approve</i>";
        echo "<th><i>Disapprove</i>";
//         echo "<th><i>Abstain or Not Voted</i>";
        echo "</tr>";

        foreach($this->tallies as $prop_id => $tallies_for_prop) {
            echo "<tr>";
            $text = $poll->props[$prop_id]->text;
            echo "<td>";
            if(! $poll->is_straw) {
                echo "<a href=\"javascript:frameset.popup_fullpropview_window($prop_id)\">";
                echo htmlspecialchars($text);
                echo "</a>";
            } else {
                echo htmlspecialchars($text);
            }
            echo "<td style=\"text-align:center;\">".$tallies_for_prop[updown_YES];
            echo "<td style=\"text-align:center;\">".$tallies_for_prop[updown_NO];
//             echo "<td>". (count($poll->userVotes) - //XXX
//                            ($tallies_for_prop[updown_YES] +
//                             $tallies_for_prop[updown_NO]));
        }
        echo "</table>";
    }

    function viewChoicesWithDetails(&$poll) {
        echo "<table border=0 class=poll_choice_summary>";
        echo "<tr style=\"text-align:left;text-decoration:underline;\">";
        echo "<th>Propositions";
        echo "<th><i>Approve</i>";
        echo "<th><i>Disapprove</i>";
//         echo "<th><i>Abstain or Not Voted</i>";
        echo "</tr>";

        foreach($this->tallies as $prop_id => $tallies_for_prop) {
            echo "<tr>";
            $text = $poll->props[$prop_id]->text;
            echo "<td>".htmlspecialchars($text);
            echo "<td style=\"text-align:center;\">".$tallies_for_prop[updown_YES];
            echo "<td style=\"text-align:center;\">".$tallies_for_prop[updown_NO];
//             echo "<td>". (count($poll->userVotes) - //XXX
//                            ($tallies_for_prop[updown_YES] +
//                             $tallies_for_prop[updown_NO]));
            //---- details.
            echo "<tr>";
            echo "<td colspan=3>";
            echo $poll->props[$prop_id]->makeDetailedView();
            echo "</td></tr>";
            
        }
        echo "</table>";
    }
        

    function getResultsReportText(&$poll) {
        global $drules; //please work
        
        $winners = $poll->drule->evaluate($poll);
        $s = "";
        $s .= "The decision rule was ".$poll->drule->longname.".\n";
        if(count($winners) == 0) {
            $s .= "No propositions were selected as winners.\n";
        } else if (count($winners) == 1) {
            $s .= "There was 1 winning proposition selected:\n\n";
        } else if(count($winners) > 1) {
            $s .= "There were ".count($winners)." winning propositions:\n\n";
        } else {
            assert(false);
        }

        foreach($winners as $prop) {
            $s .= "* ".$prop->text."\n";
            $s .= "\tYes: ".$this->tallies[$prop->prop_id][updown_YES]."\n";
            $s .= "\tNo: ".$this->tallies[$prop->prop_id][updown_NO]."\n";
            $s .= "\n";
        }
        return $s;
    }

    function viewUserVoteBreakdownInTable($poll, $uv) {
            echo "<tr>";
            echo "<td>".make_linked_username($uv->username).":";
            $firstTime = true;
            foreach($poll->props as $prop) {
                if(!$firstTime) echo "<tr><td>";
                else            $firstTime = false;

                if(count($poll->props) > 1)
                    echo "<td><i>".$prop->text."</i>";
                else
                    echo "<td> &nbsp;</td>";
                $choice = $uv->vote_data[$prop->prop_id];
                echo "<td>".updown_choiceAsHtml($choice);
            }
            echo "</tr>";
            if($uv->msgtext) {
                echo "<tr><td colspan=3>Comment: ";
                echo "<i>" .htmlspecialchars($uv->msgtext). "</i>";
            }
    }

    function viewAsBreakdown(&$poll) {
        /* Grouped by user */
        echo "<table border=0 class=poll_choice_breakdown><tr><td>";
        echo "<span class=poll_breakdown_title>Member choices/comments:</span>";
        echo "<table class=poll_breakdown_entries>";
        foreach($poll->userVotes as $uv) {
            if(!$uv->hasVoted()) continue;
            $this->viewUserVoteBreakdownInTable($poll, $uv);
            echo "<tr><td>&nbsp;</tr>";
        }
        echo "</table>";

        /* Grouped by proposition */
        if(count($poll->props) > 1) {
            echo "<span class=poll_breakdown_title>Breakdown by proposition:</span>";
            
            echo "<table class=poll_breakdown_entries>";

            foreach($this->tallies as $prop_id => $tallies_for_prop) {
                echo "<tr>";
                echo "<th colspan=2>";
                echo $poll->props[$prop_id]->text;
                $uservotes = $poll->userVotes;
                foreach(array(updown_YES, updown_NO) as $choice) {
    //                 echo "<BR>Loop on $choice <BR>";
                    foreach($uservotes as $uv) {
                        if($uv->vote_data[$prop_id] == $choice) {
                            echo "<tr>";
                            echo "<td>";
                            echo updown_choiceAsHtml($choice) . ":";
                            echo "<td>";
                            echo make_linked_username($uv->username);
                            echo "<td style=\"font-size:80%;\">";
                            echo strftime("%b-%d,&nbsp;%R", strtotime($uv->when_voted));
                        }
                    }
                }
            }
            echo "<tr><td>&nbsp;<td>&nbsp;";
            echo "</table>";
        }

        /* Those who have not voted. */
        echo "<span class=poll_breakdown_title>";
        echo $poll->isOpen() ? "Have not voted:" : "Did not vote:";
        echo "</span>";
        echo "<BR>";
        
        $have_not_voted = array();
        foreach($poll->userVotes as $uv) 
            if(!$uv->hasVoted()) 
                $have_not_voted[] = make_linked_username($uv->username);

        echo implode(", ", $have_not_voted);
            
        echo "</table>";
    }

    function viewAsBallotOptions(&$poll, &$uservote) {
        //XXX form validation - must vote for each one!!!
        //currently it's not the end of the world, registers "Has Not Voted"
        echo "<table class=poll_choice_summary><tr><td>";
        echo "<table>";
        echo "<tr><td>Proposition";
        echo "<td> &nbsp; ".updown_choiceAsHtml(updown_YES)." </td>";
        echo "<td> &nbsp; ".updown_choiceAsHtml(updown_NO)." </td>";
        foreach($poll->props as $prop_id => $prop) {
            echo "<tr>";
            echo "<td>";
            if(! $poll->is_straw) {
                echo "<a href=\"javascript:frameset.popup_fullpropview_window($prop_id)\">";
                echo "<b>".htmlspecialchars($prop->text)."</b>";
                echo "</a>";
            } else {
                echo "<b>".htmlspecialchars($prop->text)."</b>";
            }

            foreach(array(updown_YES, updown_NO) as $choice) {
                echo "<td style=\"text-align:center;\">";
                $is_chosen_now = ($choice == $uservote->vote_data[$prop->prop_id]);
                echo "<input type=radio name=prop_id_{$prop->prop_id} value=$choice ";
                echo $is_chosen_now ? "checked=checked " : "";
                echo ">";
            }
            if(! $poll->is_straw) {
                echo "<tr><td colspan=3>";
                echo "<i>Implementation details:</i> ";
                echo $prop->getImplementationDetails();
            }
        }
        echo "</table>";
        echo "</table>";
    }

}



function makeDefaultVoteData_updown($prop_ids) {
//    global $updown_NOVOTEYET; //wtf?????? *does* work from marea/newpoll but does NOT work from lib/Poll ????!!!?!!?!?!?!?!?!  The ONLY difference is, this funciton is called from within a method on Poll, but from newpoll it's called from the global context.  WHAT THE HELL?  IT GETS WIPED OUT IN DIFFERENT CONTEXTS??  GLOBAL IS NOT ACTUALLY GLOBAL?????!!!  But it knows it exists, else if it was never definted you'd get a warning!  Instead, it's NULL instead of  the correct -1!  CURSE PHP AHHHHAHAHAHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
    //next day, 12/29: switched to define() constants instead.  Works fine.


    $r = array();
    foreach($prop_ids as $prop_id) {
        $r[$prop_id] = updown_NOVOTEYET;
    }
    return $r;
}


class UserVote_updown extends UserVote {
    /* vote_data is just an array of  prop_id => currentvotestatus */
    function processVoteSubmission($db, &$post) {
        //see Tally_updown::viewAsBallot.
        //each key/val pair looks like prop_id_XXX => -1|0|1 (a choice)
        foreach($post as $k => $v) {
            $matches=array();
            if(!preg_match('/^prop_id_(\d+)$/', $k, $matches)) continue;
            $this->vote_data[$matches[1]] = $v;
        }
        printr($this);

        $ser = addslashes(serialize($this->vote_data));
        $q = "UPDATE userpolls SET vote_data = '$ser',
                                   when_voted = NOW()
              WHERE username = '".$this->username."' AND
                    poll_id  =  ".$this->poll_id;
        $db->query($q);
    }
    function viewCurrentVote(&$poll) {
        assert($this->hasVoted());

        echo "You're currently voting for:";
        echo "<table>";
        $poll->tally->viewUserVoteBreakdownInTable($poll, $this);
        echo "</table>";
        echo "<span style=\"font-size:30%;\">&nbsp;</span><BR>";
        echo "<a class=vote_button href=\"";
        echo $_SERVER['REQUEST_URI'] . "&as_ballot=1";
        echo "\">Change your vote</a>";
        echo "<BR>";
        echo "<span style=\"font-size:30%;\">&nbsp;</span><BR>";
    }
}




/* ------------------ end method updown ---------------- */


/* --------------------------------------------------------------------------
 * DRules.   naming scheme:  $method_$druleofmethod
 */

    


class DRule_updown_consen {
    function evaluate(&$poll) {
        $r = Array();
        foreach($poll->tally->tallies as $prop_id => $yesnoCounts)
            if( $yesnoCounts[updown_YES] > 0 && $yesnoCounts[updown_NO] == 0 )
                $r[$prop_id] = $poll->props[$prop_id];
        return $r;
    }
}

class DRule_updown_consenminus1 {
    function evaluate(&$poll) {
        $r = Array();
        foreach($poll->tally->tallies as $prop_id => $yesnoCounts)
            if( $yesnoCounts[updown_YES] > 0 && $yesnoCounts[updown_NO] <= 1 )
                $r[$prop_id] = $poll->props[$prop_id];
        return $r;
    }
}



// @param $yesnoCounts is from the values of the tallies assoc array,
//        looks like  {updown_YES => 5, updown_NO => 3}
function _updown_checkPropForThresh($yesnoCounts, $yesPercentageThresh, $strict)
{
    $total = $yesnoCounts[updown_YES] + $yesnoCounts[updown_NO];
    $neededYeses = $total * $yesPercentageThresh;
    return ($yesnoCounts[updown_YES]  > $neededYeses || 
             ($yesnoCounts[updown_YES] == $neededYeses && !$strict)
            ) && 
            $yesnoCounts[updown_YES] > 0;
}

class DRule_updown_majority {
    function evaluate(&$poll) {
        $r = Array();
        foreach($poll->tally->tallies as $prop_id => $yesnoCounts)
            if(_updown_checkPropForThresh($yesnoCounts, 0.5, true))
                $r[$prop_id] = $poll->props[$prop_id];
        return $r;
    }
}

class DRule_updown_twothird {
    function evaluate(&$poll) {
        $r = Array();
        foreach($poll->tally->tallies as $prop_id => $yesnoCounts)
            if(_updown_checkPropForThresh($yesnoCounts, 0.666666666666666, false))
                $r[$prop_id] = $poll->props[$prop_id];
        return $r;
    }
}

class DRule_updown_threefourth {
    function evaluate(&$poll) {
        $r = Array();
        foreach($poll->tally->tallies as $prop_id => $yesnoCounts)
            if(_updown_checkPropForThresh($yesnoCounts, 0.75, false))
                $r[$prop_id] = $poll->props[$prop_id];
        return $r;
    }
}



class DRule_chooseone_plurality {
    function DRule_chooseone_plurality($params) {
    }
    function evaluate(&$poll) {
        $max = -1;
        foreach($poll->tally->tallies as $prop_id => $proptally) {
            $max = ($proptally>$max) ? $proptally : $max;
        }

        $winners = Array();
        foreach($poll->tally->tallies as $prop_id => $proptally) {
            if($proptally == $max) {
                $winners[$prop_id] =& $poll->props[$prop_id];
            }
        }
        return $winners;
    }
}

// Argh, array_map seems to be bug-ful with the NULL callback..
// Instead make my own, a la python zip():  (moral of the story: php sucks)
function zip($a, $b)
{
    assert( count($a) == count($b) );
    $newarr = Array();
    for($i=0; $i<count($a); $i++) {
        $newarr[] = array($a[$i], $b[$i]);
    }
    return $newarr;
}
    
class DRule_chooseone_consen {
    function evaluate(&$poll) {
        $talliesAsTuples = zip(array_keys($poll->tally->tallies),
                               array_values($poll->tally->tallies));
        $withvotes = array_filter($talliesAsTuples,
                            create_function('$a','return $a[1]>0;'));
                                    
        if(count($withvotes) != 1)
            return array();

        $id = $withvotes[0][0];
//         echo "<b>$id</b>";
        return array($id => $poll->props[$id]);
    }
}

class DRule_chooseone_consenminus1 {
    function evaluate(&$poll) {
        //can't hit 2 lone dissents, or multiple >2 spots.
        $talliesAsTuples = zip(array_keys($poll->tally->tallies),
                               array_values($poll->tally->tallies));
        $singletons     = array_filter($talliesAsTuples,
                            create_function('$a', 'return $a[1]==1;'));
        $doublemoretons = array_filter($talliesAsTuples,
                            create_function('$a', 'return $a[1]>1;'));
        
        if(count($doublemoretons) == 1 && count($singletons) <= 1) 
        {
            $the_doublemoreton = array_pop($doublemoretons);
            return array($the_doublemoreton[0] => 
                         $poll->props[$the_doublemoreton[0]]);  
        }
        if(count($doublemoretons) == 0 && count($singletons) == 1)  {
            $the_singleton = array_pop($singletons);
            return array($the_singleton[0] => 
                         $poll->props[$the_singleton[0]]);
        }
        if(count($doublemoretons) > 1) 
            return array();
        if(count($singletons) > 1) 
            return array();
        if(count($doublemoretons) == 0 && count($singletons) == 0) 
            return array();
        trigger_error("fell thru the conditional tests on chooseonw_consenminus1::evaluate");
        return array();
    }
}

function threshChoiceEvaluate(&$poll, $fraction) 
{
    $totalvotes = array_sum($poll->tally->tallies);
    $thresh = ceil($totalvotes * $fraction);
    if($thresh==0) $thresh=1;
    if(debug_mode()) {
        echo "DEBUG: need thresh: $thresh from fraction: $fraction<BR>";
    }
    $talliesAsTuples = zip(array_keys($poll->tally->tallies),
                           array_values($poll->tally->tallies));
    //Can't use aarray_filter b/c no lexical scoping (grrrr)
//     printr($talliesAsTuples);
    $winnerTuples = array();
    foreach($talliesAsTuples as $tuple)
        if($tuple[1] >= $thresh) 
            $winnerTuples[] = $tuple;

    $r = array();
    foreach($winnerTuples as $tuple) {
        $id = $tuple[0];
        $r[$id] = $poll->props[$id];
    }
    return $r;
}

class DRule_chooseone_majority {
    function evaluate(&$poll) {
        $have50percent =& threshChoiceEvaluate($poll, 0.5);
        //Note: this could have 2 if all votes are split dead even 
        // between 2 separate props..
        if(count($have50percent) == 2)
            return array();
        if(count($have50percent) > 2) 
            trigger_error("failed sanity check: multiple majorities");

        return $have50percent;
    }
}
class DRule_chooseone_twothird {
    function evaluate(&$poll) {
        return threshChoiceEvaluate($poll, 0.66666666666666666666666666);
    }
}
class DRule_chooseone_threefourth {
    function evaluate(&$poll) {
        return threshChoiceEvaluate($poll, 0.75);
    }
}

/* ------------ END DRULE and METHOD IMPLEMENTATIONS CODE ------------------ */

/* UserVotes should be constructed with a factory function */
class UserVote {
    function hasVoted() {
        return $this->when_voted != '0000-00-00 00:00:00';
    }
}
/* The factory. */
function newUserVoteFromRow(&$row) {
//     echo "starting newuservotefromrow";
//     printr($row);
    eval('$uv = new UserVote_'.$row['method_type'].';');
    foreach($row as $k => $v) {
        $uv->$k = $v;
    }
    $uv->vote_data = unserialize($row['vote_data']);
    return $uv;
}
function fetchUserVote($db, $username, $poll_id)
{
    $q = "SELECT * FROM userpolls WHERE username = '$username' AND
                                        poll_id  = $poll_id";
    $row = $db->getRow($q);
    assert($row);
    $uv = newUserVoteFromRow($row);
    return $uv;
}

// XXX why the heck do i have druleParams in here?  grep -ri'ing for
// druleparams and drule_params show that newpoll.php submit,
// and here both repsect it, but nothing creates it in the first place??
function newDRule($methodType, $druleType, $druleParams=array())
{
    global $drules;
    $classname = "DRule_{$methodType}_{$druleType}";
    eval('$d = new '.$classname.'($druleParams);');
    $d->longname = $drules[$druleType]['longname'];
    $d->method_type = $methodType;
    $d->drule_type = $druleType;
    return $d;
}

function newTally($methodType, &$methodParams, &$prop_ids)
{
    $classname = "Tally_{$methodType}";
    eval('$t = new '.$classname.'($methodParams, $prop_ids);');
    return $t;
}


//
// /* Now generate stubs for all the unimplemented DRule's. */
//
// /* $stubhash will be of format
//  * $methodname => ($drulename => (constructor_code => [string],
//  *                                eval_code        => [string]))
//  */
// $stubhash = array();
// foreach($pollingMethods as $method) {
//     $stubhash[$method['name']] = array();
//     foreach($method['drules'] as $drulename) {
//         $stubhash[$method['name']][$drulename] = array();
//         $stubhash[$method['name']][$drulename]['constructor_code'] =
//             '$this->params = $params;';
//         $stubhash[$method['name']][$drulename]['eval_code'] =
//             'return -1;';
//     }
// }
// $s = "";
//
// foreach($stubhash as $methodname => $methods) {
//     if(!isset($implementedMethods[$methodname])) {
//         $cname = "Tally_${methodname}";
//         $s .= "\n class $cname {";
//         $s .= "\nfunction $cname(\$params, \$prop_ids) {}";
//         $s .= "\nfunction tallyNewVote(&\$old, &\$new) {}";
//         $s .= "}";
//     }
//     foreach($methods as $drulename => $codearr) {
//         $name = "${methodname}_${drulename}";
//         if(isset($implementedDRules[$name])) {
//             continue;
//         }
//         $cname = "DRule_${name}";
//         $s .= "\n class $cname  {";
//         $s .= "\nfunction $cname (\$params) {";
//         $s .= $codearr['constructor_code'];
//         $s .= "}";
//         $s .= "\nfunction evaluate(&\$poll) {";
//         $s .= $codearr['eval_code'];
//         $s .= "}";
//         $s .= "}\n";
//     }
// }
//
//
// if(isset($_REQUEST['debug'])) {
//    //  echo "All stub DRules:<BR>";
// //     echo "<PRE>".$s."</pre>";
// }
//
// eval($s);



