<?php
// $Id: DemeAuth.php 74 2005-03-26 07:49:53Z brendano $

/* Deme's Authentication system
 * it's a hacked up version of the messy PEAR Auth package
 * theoretically, it should be possible to add new authentication mechanisms
 * (beware that not all of its features are used, for example, user addition)
 * At the very least, hopefully you can keep changes for your own user system
 * local to this file or others in AuthContainer/ and Container.php
 *
 *
 */
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2003 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Martin Jansen <mj@php.net>                                  |
// +----------------------------------------------------------------------+
//
// $Id: DemeAuth.php 74 2005-03-26 07:49:53Z brendano $
//

require_once "../lib/PEAR.php";
require_once "../lib/dbinit.php";


define("AUTH_IDLED",       -1);
define("AUTH_EXPIRED",     -2);
define("AUTH_WRONG_LOGIN", -3);
define("AUTH_USER_NOBODY", "nobody");


//stupid class vars can only be assigned to constants, can't be assigned to
//global vars.  so let's define a constant for the DSN.

define("DEMEAUTH_DSN",
       "mysql://".
       $CONF['DBUSER'].":".$CONF['DBPASS']."@".$CONF['DBHOST']."/".$CONF['META_DBNAME']);

    
class DemeAuth
{
    //change or throw me out, if you want a new authentication scheme
    var $params = Array(
        "dsn" =>  DEMEAUTH_DSN, 
        "table" => "users",
        "usernamecol" => "username",
        "passwordcol" => "password_md5");

    /**
     * Auth lifetime in seconds
     *
     * If this variable is set to 0, auth never expires
     *
     * @var  integer
     * @see  setExpire(), checkAuth()
     */
    var $expire = 0;

    /**
     * Has the auth session expired?
     *
     * @var   bool
     * @see   checkAuth(), drawLogin()
     */
    var $expired = false;

    /**
     * Maximum time of idleness in seconds
     *
     * The difference to $expire is, that the idletime gets
     * refreshed each time, checkAuth() is called. If this
     * variable is set to 0, idle time is never checked.
     *
     * @var integer
     * @see setIdle(), checkAuth()
     */
    var $idle = 0;

    /**
     * Is the maximum idletime over?
     *
     * @var boolean
     * @see checkAuth(), drawLogin();
     */
    var $idled = false;

    /**
     * Storage object
     *
     * @var object
     * @see Auth(), validateLogin()
     */
    var $storage = "";

    /**
     * Function defined by the user, that creates the login screen
     *
     * @var string
     */
    var $loginFunction = "";

    /**
     * Should the login form be displayed?
     *
     * @var   bool
     * @see   setShowlogin()
     */
    var $showLogin = true;

    /**
     * Current authentication status
     *
     * @var string
     */
    var $status = "";

    /**
     * Username
     *
     * @var string
     */
    var $username = "";

    /**
     * Password
     *
     * @var string
     */
    var $password = "";

    /**
     * Login callback function name
     *
     * @var string
     * @see setLoginCallback()
     */
    var $loginCallback = "";

    /**
     * Logout callback function name
     *
     * @var string
     * @see setLogoutCallback()
     */
    var $logoutCallback = "";
    
    /**
     * metaOnly boolean
     * 
     * @var boolean
     * This var denotes not to login to a specific group.
     */
    var $metaOnly = false;

    // {{{ Constructor

    /**
     * Constructor
     *
     * Set up the storage driver.
     *
     * @param string    Type of the storage driver
     * @param mixed     Additional options for the storage driver
     *                  (example: if you are using DB as the storage
     *                   driver, you have to pass the dsn string here)
     *
     * @param string    Name of the function that creates the login form
     * @param boolean   Should the login form be displayed if neccessary?
     * @return void
     */

    function DemeAuth ($loginFunction = "", $showLogin = true, $metaOnly = false) {
        
        //XXX change this if you're not using DEME's native user system
        $storageDriver = "DB";
        $options = $this->params;
        
        if ($loginFunction != "" && function_exists($loginFunction)) {
            $this->loginFunction = $loginFunction;
        }
        
        if (is_bool($showLogin)) {
            $this->showLogin = $showLogin;
        }
        
        if (is_bool($metaOnly)) {
            $this->metaOnly = $metaOnly;
        }
        $this->storage = $this->_factory($storageDriver, $options);
        //This is for local usage
        if (!$this->metaOnly) {
            global $CURGROUP;
            $this->localgroupdb = $CURGROUP->getDB();
        }
        $this->isGuest = false;
    }

    // }}}
    // {{{ _factory()

    /**
     * Return a storage driver based on $driver and $options
     *
     * @access private
     * @static
     * @param  string $driver  Type of storage class to return
     * @param  string $options Optional parameters for the storage class
     * @return object Object   Storage object
     */
    function _factory($driver, $options = "")
    {
        $storage_path = "../lib/AuthContainer/" . $driver . ".php";
        $storage_class = "Auth_Container_" . $driver;

        require_once $storage_path;

        return new $storage_class($options);
    }

    // }}}
    // {{{ assignData()

    /**
     * Assign data from login form to internal values
     *
     * This function takes the values for username and password
     * from $HTTP_POST_VARS and assigns them to internal variables.
     * If you wish to use another source apart from $HTTP_POST_VARS,
     * you have to derive this function.
     *
     * @access private
     * @global $HTTP_POST_VARS
     * @see    Auth
     * @return void
     */
    function assignData()
    {
        $post = &$this->_importGlobalVariable("post");

        if (isset($post['username']) && $post['username'] != "") {
            $this->username = $post['username'];
        }

        if (isset($post['password']) && $post['password'] != "") {
            $this->password = $post['password'];
        }
    }

    function assignDataAsNotLoggedIn()
    {
        $this->username = '';
        $this->password = '';
    }

    // }}}
    // {{{ start()

    /**
     * Start new auth session
     *
     * @access public
     * @return void
     */
    //REMEMBER: php requires ALL args to be fulfilled, even if some are
    //defaults!!!!
    function start($allowNotLoggedIn=false,
                   $allowGroupGuest=false, 
                   $overrideSandboxLeniency=false) {
        global $CURGROUP;
        if(isset($CURGROUP) && $CURGROUP->getConfigOption('sandbox') && !$overrideSandboxLeniency) {
            //XXX RESTRUCTURE: something is wrong with sandboxing 
            
            //also an automatic authentication as guest. this requires 
            //a said guest account.
            if(!$allowGroupGuest && !$allowNotLoggedIn && !$this->getAuth() && 
               !isset($_POST['username']) && !isset($_POST['password'])) {
                //echo "Putting you in as the guest:guest acct<BR>";
                $this->username = 'guest';
                $this->password = '';
                $_POST['username'] = 'guest';
                $_POST['password'] = '';
            }
        }
        if($allowNotLoggedIn) {
            $this->assignDataAsNotLoggedIn();
        } else {
            $this->assignData();
        }
        
        session_start();
        
        // bizarre.. on php 4.1.2 Apache 1.3  this was failing:
        //        if (!$this->checkAuth()) {
        // needed === false instead...
        
        if (!$this->getAuth() && !$allowNotLoggedIn) {
            
	  $this->login();
        } else if(!$this->metaOnly && !$this->getMembershipInGroup() && !$allowGroupGuest ) {
            $this->offerJoinGroup();
        }
    }
    
    function offerJoinGroup() 
    {
        global $CURGROUP;
        $cururl = urlencode($_SERVER['REQUEST_URI']);
        $url = $CURGROUP->getUrlRoot()."/main/joingroup.php?"."redirect=".$cururl."&prompt=1";
//         echo $url ."<BR><BR>";
        header("Location: $url");
        exit;
    }

    function isGroupOwner($groupname) {
        /* $q = "SELECT is_owner FROM users WHERE users.username = $username";
         *  $x = $db->getOne($q);
         *  return $x;*/

        $q = "SELECT * FROM user_permissions
              WHERE groupname = '".$groupname."' AND
              BINARY username = '" .$this->getUsername(). "' AND
              permission = 'owner'";
//         printr($q);
        $res = $this->storage->query($q);
        if(DB::IsError($res)) {
            printr($res);
        }
        return $res->numRows() > 0;
    }



    // }}}
    // {{{ login()

    /**
     * Login function
     *
     * @access private
     * @return void
     */
    function login()
    {
        $login_ok = false;

        /**
         * When the user has already entered a username,
         * we have to validate it.
         */
        if (!empty($this->username)) {
            if (true === $this->storage->fetchData($this->username, $this->password)) {
                $login_ok = true;
            } else {
                error_log("fetchData said No");
		//echo "lkj";
//                 printr($this);
            }
        }
        
        if (!empty($this->username) && $login_ok) {
            $this->setAuth($this->username);
            
            require_once "../lib/users.php";
            require_once "../lib/dbinit.php";
            $metadb = dbinit_meta_connect();
            users_setUserOptionsForSession($metadb, $this->username);
            
            //We don't use this for pod
            if (!empty($this->loginCallback)) {
            	call_user_func($this->loginCallback,$this->username);
            }
        }
        
        /**
         * If the login failed or the user entered no username,
         * output the login screen again.
         */
        if (!empty($this->username) && !$login_ok) {
            $this->status = AUTH_WRONG_LOGIN;
        }

        if ((empty($this->username) || !$login_ok) && $this->showLogin) {
//            echo "HERE WE GO TO DRAWLOGIN"; die();
            $this->drawLogin($this->storage->activeUser);
            return;
        }
    }

    // }}}
    // {{{ setExpire()

    /**
     * Set the maximum expire time
     * 
     * @access public
     * @param  integer time in seconds
     * @param  bool    add time to current expire time or not
     * @return void
     */
    function setExpire($time, $add = false)
    {
        if ($add) {
            $this->expire += $time;
        } else {
            $this->expire = $time;
        }
    }

    // }}}
    // {{{ setIdle()

    /**
     * Set the maximum idle time
     * 
     * @access public
     * @param  integer time in seconds
     * @param  bool    add time to current maximum idle time or not
     * @return void
     */
    function setIdle($time, $add = false)
    {
        if ($add) {
            $this->idle += $time;
        } else {
            $this->idle = $time;
        }
    }
    
    // }}}
    // {{{ setSessionname()

    /**
     * Set name of the session to a customized value.
     *
     * If you are using multiple instances of PEAR Auth
     * on the same domain, you can change the name of
     * session per application via this function.
     *
     * @access public
     * @param  string New name for the session
     * @return void
     */
    function setSessionname($name = "PHPSESSID")
    {
        @session_name($name);
    }

    // }}}
    // {{{ setShowLogin()

    /**
     * Should the login form be displayed if neccessary?
     *
     * @access public
     * @param  bool    show login form or not
     * @return void
     */
    function setShowLogin($showLogin = true)
    {
        $this->showLogin = $showLogin;
    }

    /**
     * Register a callback function to be called on user login.
     * The function will receive a single parameter, the username.
     *
     * @access public
     * @param  string  callback function name
     * @return void
     * @see    setLogoutCallback()
     */
    function setLoginCallback($loginCallback)
    {
        $this->loginCallback = $loginCallback;
    }

    /**
     * Register a callback function to be called on user logout.
     * The function will receive a single parameter, the username.
     *
     * @access public
     * @param  string  callback function name
     * @return void
     * @see    setLoginCallback()
     */
    function setLogoutCallback($logoutCallback)
    {
        $this->logoutCallback = $logoutCallback;
    }
        
    // }}}
    // {{{ setAuthData()
    
    /**
     * Register additional information that is to be stored
     * in the session.
     *
     * @access public
     * @param  mixed  Additional information. This parameter can
     *                have any type (integer, string, array etc).
     * @return mixed  Previous value.
     */
    function setAuthData($data)
    {
        $session = &DemeAuth::_importGlobalVariable("session");

        if (isset($session['auth']['data'])) {
            $olddata = $session['auth']['data'];
        } else {
            $olddata = null;
        }
        $session['auth']['data'] = $data;
        return $olddata;
    }
    
    // }}}
    // {{{ getAuthData()
    
    /**
     * Get additional information that is stored in the session.
     *
     * @access public
     * @return mixed  Additional information.
     */
    function getAuthData($data)
    {
        $session = &Auth::_importGlobalVariable("session");

        if (isset($session['auth'][$data])) {
            return $session['auth'][$data];
        } else {
            return null;
        }        
    }
    
    // }}}
    // {{{ setAuth()

    /**
     * Register variable in a session telling that the user
     * has logged in successfully
     *
     * @access public
     * @param  string Username
     * @param  mixed  [Deprecated] Additional information that
     *                is stored in the session. This parameter
     *                can have any type (integer, string, array
     *                etc).
     * @return void
     */
    function setAuth($username, $data = null)
    {
        if (!isset($_SESSION['auth']) && !isset($_SESSION)) {
            session_register("auth");
        }

        if (!isset($_SESSION['auth']) || !is_array($_SESSION['auth'])) {
            $_SESSION['auth'] = array();
        }

        $_SESSION['auth']['registered'] = true;
        $_SESSION['auth']['username']   = $username;
        $_SESSION['auth']['timestamp']  = time();
        $_SESSION['auth']['idle']       = time();

        if (!empty($data)) {
            DemeAuth::setAuthData($data);
        }
    }

    // }}}
    // {{{ checkAuth()

    /**
     * Checks if there is a session with valid auth information.
     *
     * @access private
     * @return boolean  Whether or not the user is authenticated.
     */
    function checkAuth()
    {
        $session = &$this->_importGlobalVariable("session");

        if (isset($session['auth'])) {

            /** Check if authentication session is expired */
            if ($this->expire > 0 &&
                isset($session['auth']['timestamp']) &&
                ($session['auth']['timestamp'] + $this->expire) < time()) {

                $this->logout();
                $this->expired = true;
                $this->status = AUTH_EXPIRED;

                return false;
            }

            /** Check if maximum idle time is reached */
            if ($this->idle > 0 &&
                isset($session['auth']['idle']) &&
                ($session['auth']['idle'] + $this->idle) < time()) {

                $this->logout();
                $this->idled = true;
                $this->status = AUTH_IDLED;

                return false;
            }

            if (isset($session['auth']['registered']) &&
                isset($session['auth']['username']) &&
                $session['auth']['registered'] == true &&
                $session['auth']['username'] != "") {

                DemeAuth::updateIdle();

                return true;
            }
        }

        return false;
    }

    /* @access public
     */
    
    function getMetaOnly () {
      return $this->metaOnly;
    }
    
    function setMetaOnly ($metaOnly) {
      $this->metaOnly = $metaOnly;
      if(!$metaOnly) {
          global $CURGROUP;
          $this->localgroupdb = $CURGROUP->getDB();
      }
    }
    
    function getMembership($marea_id) {
        assert($this->getAuth());
        if ($this->metaOnly) return true;
        $q = "SELECT perms_type FROM mareas WHERE marea_id = $marea_id";
        $res = $this->localgroupdb->query($q);
        $row = $res->fetchRow();
        $perms_type = array_pop($row);
        
        if ($perms_type == "everyone") {
            return true;
        }
        
        $q = "SELECT marea_id FROM usermareas
              WHERE marea_id = $marea_id AND
                    BINARY username = '" .$this->getUsername(). "'";
        $res = $this->localgroupdb->query($q);
        return $res->numRows() > 0;
    }

    function getMembershipInGroup()
    {
        global $CURGROUP;
        //another crude sandbox hack
//         if($CURGROUP->getConfigOption('sandbox')) return true;
        
        if(!$this->getAuth()) return false;
//         assert($this->getAuth());
	if($this->metaOnly) return true;
        $q = "SELECT * FROM user_permissions
              WHERE groupname = '".$CURGROUP->getName()."' 
              AND   BINARY username = '" .$this->getUsername(). "'";
//         printr($q);
        $res = $this->storage->query($q);
        if(DB::IsError($res)) {
            printr($res);
        }
        return $res->numRows() > 0;
    }




    
    // }}}
    // {{{ getAuth()

    /**
     * Has the user been authenticated?
     *
     * @access public
     * @return bool  True if the user is logged in, otherwise false.
     */
    function getAuth()
    {
        $session = &$this->_importGlobalVariable("session");

        if (!empty($session) && 
            (isset($session['auth']['registered']) && 
             $session['auth']['registered'] === true))
        {
            return true;
        } else {
            return false;
        }
    }

    // }}}
    // {{{ drawLogin()

    /**
     * Draw the login form
     *
     * Hacked to what we want for Deme.
     *
     * @access private
     * @param  string  Username if already entered
     * @return void
     */
    function drawLogin($username = "")
    {
        global $CURGROUP, $CONF;
        //default POD setup does NOT use loginFunction.
        if ($this->loginFunction != "") {
            call_user_func($this->loginFunction, $username, $this->status);
            return;
        }

        //default DEME setup: goto login.php
        // XXX change this to a centralized getCurrentPagesUrl function out of common.php
        $cururl = urlencode(stripslashes($_SERVER['REQUEST_URI']));
        $loginurl = $CONF['DEME_URL_ROOT']."/meta/login.php?".
            "redirect=".$cururl;
                   
        if ($this->metaOnly) {
            header("Location: ".$loginurl);
        } else {
            $loginurl .= "&group=".$CURGROUP->getName();
            header("Location: ".$loginurl);
        }
        exit;
        
    }


    // }}}
    // {{{ logout()

    /**
     * Logout function
     *
     * This function clears any auth tokens in the currently
     * active session and executes the logout callback function,
     * if any
     *
     * @access public
     * @return void
     */
    function logout()
    {
//        unset($auth);
        if (!empty($this->logoutCallback)) {
            call_user_func($this->logoutCallback,$this->username);
        }
        
        $this->username = "";
        $this->password = "";

        $session = &$this->_importGlobalVariable("session");

        $session['auth'] = array();


        // WEIRD bug: if the name of the DemeAuth object is $auth in the
        // global space, then the unset() functions stick in the serialization
        // of the ENTIRE DemeAuth object INTO the session variable!

        // I blame this bug on PHP being stupid

        // It seems that no unset() or session_unregister()'s should be called
        
        if (isset($_SESSION)) {
//            error_log("isset(_SESSION)");
//            unset($session['auth']);
            //session_unregister("auth"); //Brendan: why commented out?
        } else {
//            error_log("not isset(_SESSION)");
//            unset($session['auth']);
//            session_unregister("auth");
        }        

        require_once "../lib/users.php";
        users_clearUserOptionsFromSession();
        
    }

    // }}}
    // {{{ updateIdle()

    /**
     * Update the idletime
     *
     * @access private
     * @return void
     */
    function updateIdle()
    {
        $session = &$this->_importGlobalVariable("session");
        $session['auth']['idle'] = time();
    }

    // }}}
    // {{{ getUsername()

    /**
     * Get the username
     *
     * @access public
     * @return string
     */
    function getUsername()
    {
        $session = &$this->_importGlobalVariable("session");
        if (!isset($session['auth']['username'])) {
            return "";
        }
        return $session['auth']['username'];
    }

    // }}}
    // {{{ getStatus()

    /**
     * Get the current status
     *
     * @access public
     * @return string
     */
    function getStatus()
    {
        return $this->status;
    }

    // }}}
    // {{{ sessionValidThru()

    /**
     * Returns the time up to the session is valid
     *
     * @access public
     * @return integer
     */
    function sessionValidThru()
    {
        $session = &$this->_importGlobalVariable("session");
        if (!isset($session['auth']['idle'])) {
            return 0;
        }
        return ($session['auth']['idle'] + $this->idle);
    }

    // }}}
    // {{{ listUsers()

    /**
     * BAD FUNCTION BAD BAD FOR DEME
     * List all users that are currently available in the storage
     * container
     *
     * @access public
     * @return array
     */
    function listUsers()
    {
        return $this->storage->listUsers();
    }

    // }}}
    // {{{ addUser()

    /**
     * BAD FUNCTION BAD BAD FOR DEME
     * Add user to the storage container
     *
     * @access public
     * @param  string Username
     * @param  string Password
     * @param  mixed  Additional parameters
     * @return mixed  True on success, PEAR error object on error
     *                and AUTH_METHOD_NOT_SUPPORTED otherwise.
     */    
    function addUser($username, $password, $additional = "")
    {
        return $this->storage->addUser($username, $password, $additional);
    }

    // }}}
    // {{{ removeUser()

    /**
     * Remove user from the storage container
     *
     * @access public
     * @param string Username
     * @return mixed  True on success, PEAR error object on error
     *                and AUTH_METHOD_NOT_SUPPORTED otherwise.
     */
    function removeUser($username)
    {
        return $this->storage->removeUser($username);
    }

    function changePassword($newpass) {
        $this->storage->changePassword($this->getUsername(), $newpass);
    }

    // }}}
    // {{{ _importGlobalVariable()

    /**
     * Import variables from special namespaces.
     * For pre-php4 compatibility (we shouldnt bother for Deme)
     *
     * @access private
     * @param string Type of variable (server, session, post)
     * @return array
     */
    function &_importGlobalVariable($variable) 
    {
        $var = null;

        switch (strtolower($variable)) {

            case "server" :
                if (isset($_SERVER)) {
                    $var = &$_SERVER;
                } else {
                    $var = &$GLOBALS['HTTP_SERVER_VARS'];
                }
                break;

            case "session" :
                if (isset($_SESSION)) {
                    $var = &$_SESSION;
                } else {
                    $var = &$GLOBALS['HTTP_SESSION_VARS'];
                }
                break;

            case "post" :
                if (isset($_POST)) {
                    $var = &$_POST;
                } else {
                    $var = &$GLOBALS['HTTP_POST_VARS'];
                }
                break;

            case "cookie" :
                if (isset($_COOKIE)) {
                    $var = &$_COOKIE;
                } else {
                    $var = &$GLOBALS['HTTP_COOKIE_VARS'];
                }
                break;

            case "get" :
                if (isset($_GET)) {
                    $var = &$_GET;
                } else {
                    $var = &$GLOBALS['HTTP_GET_VARS'];
                }
                break;

            default:
                break;

        }

        return $var;
    } 

    // }}}
}
?>
