<?php
/*
 * This is the class that implements the mass-emailing backend.  Currently
 * it's just a for loop over php's sendmail wrapper, mail()
 *
 * This has problems [according to dan robinson of civicactions.net]: (1) the
 * wrapper is flaky, and (2) due to HTTP timeouts it can't scale past 50-ish
 * users.  But it works well enough for our purposes for now.
 *
 * Switching to more heavy-duty backend, like a big-time mailing list engine
 * (majordomo, mailman, ezmlm) or something similar, could be implemented here
 * and hopefully with no changes to the Emailer public interface.
 *
 * phpmailer: http://phpmailer.sourceforge.net/ looks relatedly useful
 * PEAR::Mail and PEAR::Mail_Queue also mentioned in
 * http://php.net/manual/en/function.mail.php
 * a queue sounds like the right approach.
 *
 *
 * To think about: how do we go beyond plaintext mailings?
 */

class Emailer {
    
    function Emailer() {
        $this->from = "\"Deme Notifier\" <no_reply@piece.stanford.edu>";

        $this->nonuserRecipients = array();
        $this->recipients = array();

        $this->group_url = "null";
        $this->group_name = "null";
        $this->msg_id = 0;
        $this->marea_id = 0;
        $this->marea_name = null;
    }

    function setRecipientsFromRows(&$assoc_user_rows) {
        $this->recipients = Array();
        foreach($assoc_user_rows as $user_row) {
            if(isset($user_row['firstname']) && isset($user_row['lastname'])) {
                //$s =  "\"$user_row[firstname] $user_row[lastname]\" <$user_row[email]>";
                $s = $user_row['email'];
            } else {
                assert(false);
            }
            $this->recipients[] = $s;
        }
    }
    
    function setNonuserRecipientsFromRows(&$assoc_rows) {
        $this->nonuserRecipients = Array();
        foreach($assoc_rows as $nonuser_row) {
            $s = "$nonuser_row[email]";
            $this->nonuserRecipients[] = $s;
        }
    }
    function setTextMessage($msg) {
        $this->msg = $msg;
    }
    function setSubject($subj) {
        $this->subj = $subj;
    }
    function setFrom($from) {
        $this->from = $from;
    }
    function setFromByUser($metadb, $username) {
        require_once "../lib/users.php";
        $this->from = users_getFullEmailAddressFromDB($metadb, $username);
    }
        
    function setMsgId($msg_id) {
        $this->msg_id = $msg_id;
    }
    function setMareaId($marea_id) {
        $this->marea_id = $marea_id;
    }
    function setGroupName($group_name) {
        $this->group_name = $group_name;
    }
    function setGroupUrl($group_url) {
        $this->group_url = $group_url;
    }
    function setMareaName($marea_name) {
        $this->marea_name = $marea_name;
    }
    function send() {
        global $CURGROUP, $CONF;
        assert($this->msg || $this->msg === "");
        $footer = "

______________________________________________________________________
".$CURGROUP->getLongName()."
Your Mailing settings: ".$CONF['DEME_URL_ROOT']."/meta/usersettings.php?mode=group&group=".$CURGROUP->getName()."
Group page: ".$CURGROUP->getUrlRoot()."\n";
        
        $footer_unreg = "
______________________________________________________________________
".$CURGROUP->getLongName()."
You are subscribed but not fully registered for this group.
Register at ".$CURGROUP->getUrlRoot()."/main/invitee-enter.php?email=%s
";
        $body = $this->msg . $footer;
        $body_unreg = $this->msg . $footer_unreg;
        $subjline = "[".$CURGROUP->getConfigOption('email_subj_prefix')."] " . $this->subj;

        if(debug_mode()) {
            echo "\n------------------ Sending the following mail: <pre>\n";
            echo "Subject: $subjline\n";
            echo $body;
            echo "</pre>";
            echo "\n------------------ end email.<BR><BR>\n\n";
        }
        /// mareaname-replyXX@groupname.DOMAIN.COM

        if($this->marea_name === null) {
            $this->marea_name = "MAREA-NAME-HERE";
            //   error_log("ERROR: marea_name hasn't been set in Emailer!  we need a marea_name to send an email!\n");
        }
            
        $replyto_address =
            $this->marea_name . "-reply" .$this->msg_id .
            "@" . $this->group_name . $CONF['DOMAIN_SUFFIX'];
            
        $common_hdrs =
            "To: $replyto_address\n".
            "From: $this->from\n" .
//            "Cc: $replyto_address\n" .
            //"Reply-To: $replyto_address\n" .
            "X-Deme-Msg-Id: $this->msg_id\n" .
            "X-Deme-Marea-Id: $this->marea_id\n" .
            "X-Deme-Group-Name: $this->group_name\n" .
            "X-Deme-Group-Url: $this->group_url\n";

        //For detecting bounces.  Need to add to cvs conf.php.  still untested
        if( isset($CONF['RETURN_PATH']))
            $common_hdrs .= "Return-Path: ".$CONF['RETURN_PATH']."\n";
        if( isset($CONF['RETURN_RECEIPT_TO']))
            $common_hdrs .= "Return-Receipt-To: ".$CONF['RETURN_RECEIPT_TO']."\n";
        

        foreach($this->recipients as $recip) {
            if(debug_mode()) {
                echo "Emailer: ".htmlspecialchars($recip)." | $subjline <BR>\n";
            }
            //XXX i think we've hit the limits of php mail()
            //forcing $recip to be in the To: header
            //this is UNLIKE good mailing list behavior where $recip
            //does NOT appear in To: or Cc:.
            //Raw SMTP lets you be more flexible, so does python's smtp.py
            //maybe PEAR::Mail lets you???
            mail($recip, $subjline, $body, $common_hdrs);
        }
        foreach($this->nonuserRecipients as $recip) {
            if(debug_mode()) {
                echo "Emailer: ".htmlspecialchars($recip)." | $subjline <BR>\n";
            }
            $body_here = sprintf($body_unreg, urlencode($recip));
            mail($recip, $subjline, $body_here, $common_hdrs);
        }
    }
}
