<?
// $Id: Group.php 82 2005-04-26 00:35:26Z brendano $

//General functions for manipulating group-level stuff
//some is OO, some is not
//everything should move to being OO


require_once "../lib/fileutil.php";

class Group
{

    /*
     * do NOT call this constructor directly, use a factory to
     * get from database.
     */
    function Group() {
        $this->metarow = null;
        $this->db = null; //Connection to group's DB
    }


    /**
     * Gets the connection to this group's DB.
     * (and creates a connection if necessary)
     */
    function getDB() {
        assert( $this->metarow );
        if( ! $this->db) $this->connectToDB();
        return $this->db;

    }

    /**
     * Returns the shortname.
     */
    function getName() {
        assert( $this->metarow );
        return $this->metarow['name'];
    }

    /**
     * Returns the longname.
     */
    function getLongName() {
        assert($this->metarow);
        return $this->metarow['long_name'];
    }

    /**
     * The root of the url for the group.
     * e.g. http://gs.o/deme/groups/the-super-group
     * or   http://the-super-group.gs.o
     */
    function getUrlRoot() {
        global $CONF;
        if($CONF['USE_SUBDOMAIN_SCHEME']) {
            return "http://".$this->getName().".". $CONF['DOMAIN_SUFFIX'];
        } else {
            return $CONF['DEME_URL_ROOT'] . "/groups/".$this->getName();
        }
    }

    /**
     * With a trailing slash, saves an http redirect
     * e.g. http://gs.o/deme/groups/the-super-group/
     */
    function getTopUrl() {
        return $this->getUrlRoot() . "/";
    }

    /**
     * Returns the path to the serverside data subdirectory for this group.
     */
    function getDataDir() {
        global $CONF;
        return $CONF['DEME_DATA_DIR'] . "/" . $this->getName();
    }

    function getEmailSubjPrefix() {
        $db = $this->getDB();
        $esp = $db->getOne("SELECT email_subj_prefix FROM groupopts");
        return $esp;
    }



    function getConfigOption($optname) {
        $db = $this->getDB();
        return $db->getOne("SELECT $optname FROM groupopts");
    }

    function setConfigOption($optname, $newval) {
        $db = $this->getDB();
        $db->query("UPDATE groupopts SET $optname = '".addslashes($newval)."'");
    }


    /**
     * private-ish.  doesn't return anything.   getDB() is preferred.
     */
    function connectToDB() {
        $this->db = Group::_getDBConnection($this->metarow['name']);

    }

    /**
     * private-ish, STATIC.  returns DB connection.
     */
    function _getDBConnection($groupname) {
        global $CONF;
        return dbinit_connect(
            $CONF['DBUSER'], $CONF['DBPASS'],
            $CONF['DBHOST'], Group::makeDbName($groupname));   
        
    }

    function makeDbName($groupname) {
        global $CONF;
        return $CONF['GROUP_DB_PREFIX'] ."_" .$groupname;
    }
    /**
     * This is a STATIC factory.  Use this instead of a constructor:
     *
     * $group = Group::fetchGroupByName($metadb, $name)
     *
     * @param $name: the group's short name.
     * @return the row in the groups database.
     */
    function fetchGroupByName($metadb, $name) {
        $q = "SELECT * FROM groups WHERE name = '".addslashes($name)."'";
        $res = $metadb->getAll($q);
        if( count($res) == 0) {
            trigger_error("No groups with name '$name'");
            return null;
        }
        if( count($res) > 1) {
            trigger_error("More than one group with name '$name'");
            return null;
        }

        $newgroup = new Group();
        $newgroup->metarow = $res[0];
        return $newgroup;
    }

    /**
     * This is a STATIC factory.  unlike fetchGroupByName, it does
     * NOT get anything from the database, so it only can create
     * a hollow shell of a "group".
     *
     * The resultant Group is useful for file/pathname stuff,
     * but using it for real things will fail.
     */ 
    function createFakeGroup($name) {
        $metarow = array('name' => $name);
        $newgroup = new Group();
        $newgroup->metarow = $metarow;
        return $newgroup;
    }

    /**
     * STATIC.  
     * This actually CREATES a brand new group in the DB and filesystem,
     * then fetches it right back out,
     * returning the new group on success, null on failure
     */
    function createNewGroup($name, $long_name, $email_subj_prefix) {
        global $CONF;
        $metadb = dbinit_meta_connect();

        //Update the meta db with info
        $long_name_sql = addslashes($long_name);
        $sql = "INSERT groups SET name='$name', long_name='$long_name_sql'";
        $res = $metadb->query($sql);

        //Create and fill the new group database
        $dbname = Group::makeDbName($name);

        //XXX should check for error on group creation
        //but maybe have to change the pear error callback for that?
        //right now it callbacks out and fatal errors.
        $sql = "CREATE DATABASE `$dbname`";
        $res = $metadb->query($sql);

        dbinit_run_dumpfile($dbname,
                            "../offweb/group.skel.sql");

//        $db = Group::_getDBConnection($name);
//        $sql = file_get_contents("../offweb/group.skel.sql");
//        $res = $db->query($sql);
        
        //Create and fill the new group's filesystem subdir
        
        $success = copyr("../offweb/groupdata-skel", $CONF['DEME_DATA_DIR']."/".$name, 0777);
        if(!$success) {
            trigger_error("group data skeleton copy FAILED\n");
            return null;
        };

        //New group is now ready to go.  Fetch it out and do last config stuff
        $group = Group::fetchGroupByName($metadb, $name);
        if($group == null) return $group; //this is an error.

        $sql = "INSERT INTO groupopts SET
                 debug = 0,
                 sandbox = 0,
                 allow_guest_on_grouphome = 1,
                 email_subj_prefix= '".addslashes($email_subj_prefix)."'";
        $db = $group->getDB();
        $db->query($sql);
        
        return $group;
    }
}


function get_group_users($db)
{
    $usernames = array();
    $res = $db->query("SELECT username FROM users");
    while($row = $res->fetchRow()) {
        $usernames[] = ($row['username']);
    }
    return $usernames;
}


function get_group_users_sql($db)
{
    $usernames = get_group_users($db);
    $quoted_usernames = array();
    foreach($usernames as $un) {
        $quoted_usernames[] = "'".addslashes($un)."'";
    }
    return implode(",", $quoted_usernames);
}

//Precondition: $username already is a legit account on the meta database
function join_group($username, $db, $metadb, $isOwner = false)
{
    global $CURGROUP;
    $q = "INSERT user_permissions SET username='$username', 
                                groupname='".$CURGROUP->getName()."'
                                ";
    if ($isOwner) $q = "INSERT user_permissions SET username='$username', 
                                groupname='".$CURGROUP->getName()."',
                                permission='owner'
                                ";
    $metadb->query($q);

    $q = "INSERT users SET username='$username'";
    if ($isOwner) $q = "INSERT users SET username='$username', is_owner=1";
    $db->query($q);

    //Putting you as a voter in all open polls...
    $openpolls = $db->getAll("SELECT poll_id FROM polls LEFT JOIN 
        folio_items fi ON fi.indiv_type_id = polls.poll_id
        LEFT JOIN mareas ON mareas.perms_type = 'everyone' AND 
                  fi.marea_id = mareas.marea_id
        WHERE is_open = 1 AND type_id = 4          ");/* <<< code from alex which is buggy.  Reverting to old code
    $openpolls = $db->getAll("SELECT poll_id FROM polls 
        LEFT JOIN mareas ON mareas.perms_type = 'everyone' AND 
                  polls.marea_id = mareas.marea_id
        WHERE is_open = 1          ");*/
    
//  printr($openpolls); //Note this screws redirects on debug mode
    if($openpolls) {
        require_once "../lib/Decisions.php";
        require_once "../lib/Poll.php";
        
        $userpoll_values = Array();
        //XXX need to update!!!!
        foreach ($openpolls as $pollrow) {
            $poll_id = $pollrow['poll_id'];
            $poll = new Poll;
            $poll->fillFromDB($db, $poll_id);

            $vote_data = serialize($poll->makeDefaultVoteData());
            $userpoll_values[] = "( '$username', 
                                    $poll_id, 
                                    0,
                                    '".$poll->method_type."',
                                    '".addslashes($vote_data)."')";
        }
        $q = "INSERT userpolls (username,poll_id,when_voted,method_type,vote_data) VALUES ".join(",", $userpoll_values);
    //     echo $q."<BR>";
        $db->query($q);
    }

    //---- no nonuser_subscription-hood anymore!
    $email = $metadb->getOne("SELECT email FROM users WHERE username='$username'");
    $db->query("DELETE FROM nonuser_subscribers WHERE email='".addslashes($email)."'");



//need to bring this back, when we bring back per-marea permissions

    //     $openmareas = $db->getAll("SELECT marea_id FROM mareas WHERE perms_type='everyone'");
    //     foreach ($openmareas as $marearow) {
    //         $marea_id = $marearow['marea_id'];
    //         $usermarea_values[] = "('$_POST[username]', $marea_id)";
    //     }
    //     $q = "INSERT usermareas VALUES " . join(",", $usermarea_values);
    //     echo $q."<BR>";
    //     $db->query($q);

}

function leave_group($username, $db, $metadb)
{
    global $CURGROUP;

    $q = "DELETE FROM user_permissions WHERE username='$username' AND
                                groupname='".$CURGROUP->getName()."'
                                ";
    $metadb->query($q);
    
    $q = "DELETE FROM users WHERE username='$username'";
    $db->query($q);
    
    //Now undo all the user's current uservotes
    $q = "SELECT * FROM userpolls WHERE username='$username'";
    $uservotes = $db->getAll($q);
    foreach($uservotes as $row) {
        $oldUserVote = newUserVoteFromRow($row);
        $poll_id = $oldUserVote->poll_id;
        $poll = new Poll;
        $poll->fillFromDB($db, $poll_id);
        //make a blank ballot
        $blankUserVote = $oldUserVote; //copy
        $blankUserVote->vote_data = $poll->makeDefaultVoteData();
        $poll->tallyNewVoteThenUpdate($db, $oldUserVote, $blankUserVote);
    }

    //now trash 'em
    $q = "DELETE FROM userpolls WHERE username='$username'";
    $db->query($q);
}


