<?
// $Id: mailinput.php 132 2005-06-04 07:43:56Z brendano $
//Routines for handling incoming mail
//currently, exist as helpers for meta/mailhandler.php


// For msg body, prefer text/plain over text/html, and prefer the first over
// later attachments If only attachment is text/html msg body, run it through
// html->text conversion
// Ignore subsequent binary attachments
// QUESTION: Is the legit msg body always the first mime attachment?
// sometimes there's no message component, i know (pine sometimes complains)
// QUESTION: what about multiple text-like attachments?  Sometimes email
// forwards are done as MIME attachments, and we want those to appear inline.
// Do we extract them into the Deme msgtext?  Or compose together at viewtime?
// Do we re-attach for sending back out?  there's a content-disposition:
// inline sorta thing which means the text attachments are supposed to be
// assembled together into a page-like thing...
// QUESTION: we want to re-attach binary attachments, right?

// POSSIBILITY: hook up pine or mutt to do all the multipart/mime conversion
// into one textual message.  maybe they handle special cases better.
//  PROBLEM: pine is non-free (incompatible with GPL or any other reasonable
//  license.)  i havent used mutt enough to know if its text conversion is
//  any good... pine's is quite awesome.
// NOPE: i went ahead and did the recursive multipart mime analysis below.


//this all is written outside Mail_mimeDecode.php in hopes of abstracting it
//to work with PECL mailparse, or php.net imap_fetchstructure() in the
//future...

//If/when we change to decode_bodies => false, then full-out MIME decoding
//would take place here.

//TODO: read through the MIME RFC's: 2045, 2046, 2049 -- for more details
// to learn how to do things correctly?
// multipart/alternative vs. multipart/mixed is good to know, answers some
// of the questions above



// getPlainTextBody()  is the entry point for the recurisve MIME part
// analysis routines.  You pass it a mimePart object, such as the one returned
// by mimeDecode::decode().

function mailinput_getPlainTextBody(&$structure) {
//    print_r($structure);

    //Simple case: just a plain body, no MIME or anything.
    //I think 'parts' exists only if it's a multi-part MIME message?...
    //need to look at Mail_mimeDecode source to be sure...
    if( ! isset($structure->parts)) {
        assert(isset($structure->body));
        
        //RFC2045's introduction seems to imply that if no MIME,
        //it must be US-ASCII... but hopefully mimeDecode.php
        //has already decoded it all correctly.
        // [Counterexample: Outlook Express 6.0 specifies
        //  Content-Type: text/plain; charset="Windows-1252"]

        //Should do a US-ASCII -> UTF-8 conversion here, in that case...
        
        return $structure->body;
    }

    //I think this happens only on multipart/___ mime type (see rfc2046)
    //if not, then i misunderstand our parsing library Mail_mimeDecode
    //(not like it's documented well or anything)
    if(strtolower($structure->ctype_primary) != "multipart") {
        error("mailinput.php: ??? structure->parts exists, but not ctype multipart");
    }

    if(count($structure->parts) == 0) {
        // TODO: what the hell does this mean??  is this legal?
        error("mailinput.php: ??? zero parts?  is this legal?");
        return "";
    }

    // we know we have a  multipart  part now.  find which specialized
    // recursive function to call next on our part.  (these specialized
    // functions know how to properly recurse through subchildren; the proper
    // procedure is dependent on the semantics of the multipart subtype.)
    //  see rfc2046 section 5.1 "Multipart Media Type"

    // we only recognize two of the multipart subtypes for now....

    if(strtolower($structure->ctype_primary) ==   "multipart" &&
       strtolower($structure->ctype_secondary) == "alternative") {
        return processMultipartAlternative($structure);
    }

    //for multipart/mixed, but we'll assume other multiparts work like mp/mixed
    //which seems to be the correct behavior by rfc2046
    return processMultipartMixed($structure);

}

//Helper for processMultipart functions.  $parts is an array.
//[conceptually follows lisp/python filter(), python list comprehensions]
function collectPartsWithType(&$parts, $pri, $sec)
{
    $ret = array();
    for($i=0; $i < count($parts); $i++) {
        echo strtolower($parts[$i]->ctype_primary) ."\n";
        echo strtolower($parts[$i]->ctype_secondary) ."\n";
        if(strtolower($parts[$i]->ctype_primary) == $pri &&
           strtolower($parts[$i]->ctype_secondary) == $sec) {
            $ret[] =& $parts[$i];
        }
    }
    return $ret;
}

//Must return a text body; is allowed to recurse.
//Pass it the parent part (it will fetch out the array of children itself)
function processMultipartAlternative(&$multipartPart)
{
    //General strategy
    //Try in order of preference, which is now
    //  (1) text/plain
    //  (2) text/html
    //  (3) text/enriched  [though this should be same pref as text/html)
    //Does anyone use multipart/alternative for anything else than this?


    //the subparts of this part
    $parts =& $multipartPart->parts;

    //degenerate case.  is this even allowed?  seems to defeat purpose of
    //multipart/alternative
    if(count($parts) == 1) {
        return forceMimePartToPlainText($parts[0]);
    }

//    print_r($parts);
    echo "num parts before call: ".count($parts)."\n";
    //Try to find a text/plain
    $textplains =& collectPartsWithType($parts, "text", "plain");
    
    if($textplains) {
        //check for multiple parts ... though that should be illegal in a
        //multipart/alternative
        return forceMimePartToPlainText($textplains[0]);
    }
    
    $texthtmls =& collectPartsWithType($parts, "text", "html");
    if($texthtmls) {
        return forceMimePartToPlainText($texthtmls[0]);
    }
    
    $textenricheds =& collectPartsWithType($parts, "text", "enriched");
    if($textenricheds) {
        return forceMimePartToPlainText($textenricheds[0]);
    }

    //Other considerations: are you allowed to nest more multipart's inside?
    // if so, we may want to tweak this fn to be able to raise a failure,
    // and need to tweak recursive calls to allow for the possibility
    

    //OK... we couldn't find a mime type we know what to do with.
    error("mailinput.php: ERROR in multipart/alternative processing");
    return "ERROR IN MULTIPART/ALTERNATIVE PROCESSING";
}

//Helper to undo the annoying verboseness of the parser's api
function slashtype(&$part)
{
    return strtolower($part->ctype_primary)  .'/'.  strtolower($part->ctype_secondary);
}

//This function will have to deal with binary attachments and the like
function processMultipartMixed(&$multipartPart)
{
    $parts =& $multipartPart->parts;

    //degenerate case.  is this even legal?
    if(count($parts) == 1) {
        return mailinput_getPlainTextBody($parts[0]);
    }

    
    //else... they better put textual information in the first part!!
    //counterexamples where this will fail...
    // (1) clients that forward a mail as an attachment, not text the first
    // part will be empty or have a tiny note, and all the interesting stuff
    // will be in a later attachment of the forwarded message
    // (2) ok i forgot the 2nd i was going to write
    
    $firstpart =& $parts[0];
    return mailinput_getPlainTextBody($firstpart);
}


//Must return a plaintext message, no matter WHAT the type.  Should support
//text/plain, text/html, text/enriched ... in the future, maybe wackier
//formats like application/rtf, application/msword, application/pdf
// (and all the equivalent mimetype labels like application/vnd.ms-word,
//  application/pdf, text/pdf ... etc.)
// The choice whether to force a plaintext conversion must be made before
// calling this function.
// you are NOT allowed to pass a multipart part into here!
function forceMimePartToPlainText(&$part)
{

    // This was parsed from the  "primary/secondary"  Content-Type syntax
    $pri = strtolower($part->ctype_primary);
    $sec = strtolower($part->ctype_secondary);

    if($pri ==  "text" && $sec == "plain") {

//this should be decoded, right?
//TODO: recode into UTF-8, the default across all Deme currently...
//QUESTION: how the hell do you do that in php?  two solutions i can find...
//1)the multibyte extension: http://php.net/mbstring
//  [non-std!  but seems quite important, and therefore reasonable to require.]
//2) http://www.phpclasses.org/browse/package/1974.html
//  [a php-only encoding conversion class.  the website, annoyingly, requires
//  registration, though the code's license is free.. I (brendan) downloaded
//  a copy for now...]

// ...also have to see... what sort of processing does our MIME library do
// here?  does it interpret encodings or recode them?
                
        return $part->body;

        
    } elseif($pri == "text" && $sec == "html") {
        error("ERROR: no html support yet!");
        return "ERROR NO HTML SUPPORT YET\n";
    } elseif($pri ==  "text" && $sec == "enriched") {
        //Not quite the same as html.  it's weird
        error("ERROR: no text/enriched support yet!");
        return "ERROR NO TEXT/ENRICHED SUPPORT YET\n";


    } elseif($pri == "text") {
        //What the hell?  a text/(NOT plain, NOT html, NOT enriched) mime
        //attachment?
        //TODO: the UTF-8 encoding thing should probably still apply
        //here.. perhaps should treat like text/plain ?  what does the rfc say
        //to do?
        return $part->body;

        
    } else {
        error("ERROR: can't convert to plaintext mimetype $pri/$sec");
        return "ERROR: FAILED TO CONVERT MIMETYPE $pri/$sec\n";
    }
}



// email received is                  [gname] subjectline
// most email clients will then send  Re: [gname] subjectline
// if we didnt clean, we'd result in  Re: [gname] Re: [gname] subjectline
//
// This function cleans that up at the source, cutting off Re: [gname] before
// editing in the new Re: [gname].

function clean_subject($subj, $groupname)
{
//the thing at the start of a subjectline
//non-english mail clients can do different ones sometimes I know...
//AW: has been seen by Brendan by german-configured email clients
//are there zillions more out there in the world?  dunno...
    
//this is a regex
    $re_thing = "(Re:|AW:)";
    
    $gname_thing = "[". $groupname ."]";
    $stripped = preg_replace('/^'.$re_thing.'\s*'.preg_quote($gname_thing).'\s*/',
                             '',
                             $subj);
    //we want to put Re: back in, in accordance to post-main.php behavior
    //that uses "Re:" for deme subjectlines by default

    //do this only if there's no Re: yet

    if( ! preg_match('/^'.$re_thing.'/', $stripped)) {
        return "Re: ".$stripped;
    } else {
        return $stripped;
    }
}

//test: should both yield the same   Re: subjectline
//echo clean_subject("Re: [asdf] subjectline", "asdf") ."\n";
//echo clean_subject("Re: [asdf] Re: subjectline", "asdf") ."\n";


// This was an earlier stab at the problem, it's too complex and unecessary.
function intense_clean_subject($subj)
{
//$gname_thing = "[" .$CURGROUP->getName(). "]";
    $gname_thing = "[asdf]";
    
//just cut out all of the gname things, THEN reduce repetition in Re: tokens
    $subj = preg_replace( '/'.preg_quote($gname_thing).'/', '', $subj);
    echo $subj . "\n";
    
    
//the thing at the start of a subjectline
//non-english mail clients can do different ones sometimes I know...
//AW: has been seen by Brendan by german-configured email clients
//are there zillions more out there in the world?  dunno...

//this is a regex
    $re_thing = "(Re:|AW:)";


//delete lots of these things at the start of a subjectline.  theoretically,
//since all emails are processed through here, it should only be possible to
//have 2 of these things piled up.  Well, let's just do it for as many as are
//found.

//[ALTERNATE possible policy: delete all re_things.  Re: is just a space
// waster, right?]

//this regex says: of the first 1..N re_thing's (separated by whitespace),
//replace all of them with the first re_thing. (and a space to reclaim
//the last whitespace from the final re_thing that was deleted)
    
    $subj = preg_replace('/('.$re_thing.')\s*(('.$re_thing.'\s*)*)/', '$1 ', $subj);
    
    echo $subj ."\n";
    
    return $subj;
}


?>
