#!/usr/bin/php -q
<?
/***********************************************************************
 * The incoming mail handling script
 * The mailer should pass the incoming email as stdin to this script.
 ***********************************************************************/
//useful mini-tutorial: gvtulder.f2o.org/articles/incoming-mail/

//postfix puts our working dir where the script lives: deme/meta

// HOW TO DEVELOP AND DEBUG
//
//You kinda need to use the commandline.  This is how the mailer
// will call the script.
//   ./mailhandler.php marea-reply5@grpname.gs.o < saved-email
//
// one way to get a raw saved-email: pine's (E)xport with full (H)eaders
// .. outlook express's properties->details->msg source ... etc.
//
//OR, better yet, get the /tmp/mailin.$$ saved messages collected by this
// script.
//
//When email come in, info messages will get logged.  Watch via
// tail -f DEME_DATA_DIR/mailhandler.log
//
//you may also find useful, to tail -f your mailer's logfile.  on our
// machine, postfix logs to /var/log/maillog .
//
// usually what i do is, send an email, then if nothing gets posted, that
// means the script crashed somehow.  I open up a new xterm and run tail -f
// DEME_DATA_DIR/mailhandler.log, then send another email.  you get to see in
// realtime the pid of the mailhandler process that handles that email.  That
// means the email was saved as /tmp/mailin.PID.  Then run ./mailhandler.php
// POSTING_ADDY < /tmp/mailin.PID and you can then see the errors properly on
// the console, and do a fast debug-run-debugagain cycle.  yay!
//
// it would be nice if we could put some sort of php error callback thing to
// just log ALL errors... does php support this?  maybe php5 does?  but that
// still wouldnt be as nice as the commandline, where you can run it really
// fast via [Up] and [Enter] each time .. its much more timeconsuming to
// compose a new mail every time you try to fix a syntax error!


//NOTE on parsing libraries: currently we use Mail_mimeDecode.php
//from PEAR.  However, that library will be quite resource inefficient
//when parsing big emails with big attachments.  the PECL mailparse
//extension would be much better to use.  Unfortunately, it's a complete
//pain to figure out how to compile & install it, and it seems to want
//php >= 4.3.0 anyway (which we don't have on our development server at the
//moment)

// TODO: check out IMP Horde -- an opensource webmail system -- and see
// how they do THEIR parsing libraries.

//Also, it would be quite a pain to have to install this PECL mailparse
//extension for a Deme installation you want to have email input on.
//The installation mechanism differs from Unix vs. Windows, you often
//have to edit php.ini, etc. etc.

// TODO: don't process immediately, but rather enqueue it.  the title of
// the following PEAR class is promising: Mail_Queue.
// comments in Mailman's "post" script explain why queueing is needed:
    # Immediately queue the message for the incoming qrunner to process.  The
    # advantage to this approach is that messages should never get lost --
    # some MTAs have a hard limit to the time a filter prog can run.  Postfix
    # is a good example; if the limit is hit, the proc is SIGKILL'd giving us
    # no chance to save the message.
//That's bad, dude -- WE are using postfix!
    
//Another option is to let messages go into a POP/IMAP mailbox, and on a
//cronjob take off messages and process them.  so let the the mailer do all
//the queueing business for you.  this seems to be the predominant method for
//php systems that accept emails ... like post-by-email for weblogs (e.g. in
//wordpress), and many others?
    
require_once "../lib/common.php";
require_once "../lib/Mail/mimeDecode.php";
require_once "../lib/mailinput.php";
require_once "../lib/msg.php";
require_once "../lib/Notices.php";

/**
 * @return null if $addy is not a proper input address
 */
function parsePostingAddress($addy)
{
    global $CONF;
    echo( "ADDY: $addy\n" );
    $matches = array();
    preg_match('/(.+)-reply(\d+)@(.+)' . preg_quote($CONF['DOMAIN_SUFFIX']) . '/',
               $addy, $matches);
    if($matches) {
        $ret = array(
            'marea_short_name' => $matches[1],
            'in_reply_to' => $matches[2],
            'new_thread' => false,
            'group_name' => $matches[3]);
        return $ret;
    }

    preg_match('/(.+)@(.+)' . preg_quote($CONF['DOMAIN_SUFFIX']) . '/',
               $addy, $matches);
    if($matches) {
        $ret = array(
            'marea_short_name' => $matches[1],
            'in_reply_to' => null,
            'new_thread' => true,
            'group_name' => $matches[2]);
        return $ret;
    }
    return null;
}


//test code
//print_r(parsePostingAddress("werew_sdfa-werew-reply34@groupname.groupspace.org"));  exit;

function debugshow($str)
{
    //for when running as a mail handler and can't see output
    //what a hack.  change to logging or stdout/err redirect might be better...
    if( ! strstr($str, "\n"))
        $subj = $str;
    else
        $subj = "Debug time ".time();

    echo $str . "\n";
    //please put in your own address here!
    mail('brendano@stanford.edu', $subj, $str);
}

$errlog = null;
function error($str)
{
    global $CONF, $errlog;
    if($errlog === null) {
        $errlog = fopen($CONF['DEME_DATA_DIR']."/mailhandler.log", 'a');
    }
    echo $str . "\n";
    fwrite($errlog, getmypid().": ". $str ."\n");
}

function exit_script()
{
    global $errlog;
    if($errlog !== null) {
        fclose($errlog);
    }
    exit;
}
//**********************************************************************

//------------------ begin script actions -------------------

//if this came in via postfix, let us know we're alive
error("BEGIN mailhandler.php pid=".getmypid()." at ".date("Y-m-d H:m:s"));

//Properly configured Postfix gives us the recipient address on argv.  hurrah

if(count($_SERVER['argv']) >= 2) {
    $recipient = $_SERVER['argv'][1];
    $postingAddyComponents = parsePostingAddress($recipient);
    if( ! $postingAddyComponents ) {
        error("not a valid deme input address: $recipient");
        exit_script();
    }
} else {
    //Let's require the posting addy on the cmdline.
    error("ERROR: need to supply the posting address as the first argument");
    error("       on the command line!");
    exit_script();
}
        
// Read in the email from stdin
// This could change to fetching off a queue, POP or IMAP or unix mbox, etc.
$fd = fopen("php://stdin", "r");
$email = "";
$num_kb_read = 0;
while (!feof($fd)) {
    $email .= fread($fd, 1024);
    
    if($num_kb_read++ > 10000) {
        //TODO: Reject really big email (>10 MB ?)
        //with apologetic rejection notice
    }
}
fclose($fd);


//error($email);  exit;  //debugging our mailer config

//This should be useful for debugging: save the entire email.
$fd = fopen("/tmp/mailin." .getmypid(), "w");
fwrite($fd, $email);
fclose($fd);

////////////////////////////////////////////////////////////////////

// see lib/mailinput.php for code & comments about how to extract
// plaintext out of an email message


//---------- begin parsing headers and body -----------


$decode = new Mail_mimeDecode($email);

// TODO: check for X-Deme headers and ban a duplicate mail that came back!
$sent_by_deme = $decode->getHeaderValue("X-Deme-Msg-Id");
if($sent_by_deme) {
    $msg_id = $sent_by_deme;
    $email_message_id = $decode->getHeaderValue("Message-Id");
    error("Message originated from Deme: X-Deme-Msg-Id=$msg_id Message-Id=$email_message_id");
    exit_script();
}



// imap_rfc822_parse_adrlist() is awfully by-the-book, I don't trust it.
// (e.g. it rejects ;-separation of addresses and chokes if no <> surrounding
// addys.  but maybe this is reasonable behavior; I don't know how conformant
// email clients tend to be.)  It may be better to regex for
// marea-r-t-m42@name.gs.o somewhere inside To: or Cc: ... but meanwhile we'll
// just use this.

// XXX still not used yet, should go into those new DB columns
$to =  $decode->getHeaderValue("To");
$cc = $decode->getHeaderValue("Cc");
// $addyObjs1 = imap_rfc822_parse_adrlist($to, "");
// $addyObjs2 = imap_rfc822_parse_adrlist($cc, "");



//We now have the marea, message being replied to, and group!
//debugprint($postingAddyComponents);

//Get correct databases and Sanity checks: groupname and marea shortname exist

require_once "../lib/dbinit.php";
require_once "../lib/Group.php";
$metadb = dbinit_meta_connect();
$grouprow = $metadb->getRow("SELECT name FROM groups WHERE
name = '".$postingAddyComponents['group_name']. "'"
    );

//debugprint($grouprow);

if( ! $grouprow ) {
    error("Error: no group with name: ".$postingAddyComponents['group_name']);
    //XXX should send apology email to user
    exit_script();
}


//the "official" group-selection technique as used in groupsdir_handler.php

$CURGROUP = Group::fetchGroupByName($metadb, $grouprow['name']);
$db = $CURGROUP->getDB();


//Check if we have a real meeting area here

$marea_id = $db->getOne("SELECT marea_id FROM mareas
WHERE short_name LIKE
 '".addslashes($postingAddyComponents['marea_short_name'])."'");

if( ! $marea_id) {
    error("Error: no meeting area with shortname: ".
              $postingAddyComponents['marea_short_name']);
    //XXX should send apology email to user
    exit_script();
}


//Get responding-to message if incoming mail is a response

$is_response = isset($postingAddyComponents['in_reply_to']);

if($is_response) {
    $in_reply_to = $postingAddyComponents['in_reply_to'];

    // Sanity checks: in_reply_to exists
    if( ! $db->getOne("SELECT msg_id FROM msgs WHERE msg_id = $in_reply_to") ) {
        error("msg_id $in_reply_to does NOT exist in ".$CURGROUP->getName());

        //XXX what to do?  do we post to top level?  what would cause this
        //error in the first place?  using an old address?  in email clients
        //that automatically saves addresses and pops them up during the
        //user's typing ?  No... the msg_id will exist SOMEWHERE I bet,
        //since it's only possible that -replyXX email addys can exist
        //only for XX msg_id's that have already been created
        exit_script();
    }
} else {
    //this gives a top level post... right?
    $in_reply_to = 0;
}


//Get identity of user

// SECURITY PROBLEM: From: spoofing!  I'm not sure if there's even a decent
// stopgap solution out there...

$from =  $decode->getHeaderValue("From");

$fromObjs = imap_rfc822_parse_adrlist($from, "");
assert( count($fromObjs) == 1); //would be REALLY screwed up - multiple From addys on the email
$fromObj = $fromObjs[0];

$userAddy = $fromObj->mailbox ."@". $fromObj->host;

// Search for joined user with that email addy.
$q = "SELECT * FROM users
LEFT JOIN user_permissions up ON
  up.username = users.username AND
  up.groupname = '".addslashes($CURGROUP->getName())."'
WHERE email = '".addslashes($userAddy)."'
      ";
$rows = $metadb->getAll($q);

//print_r($rows);

// Sanity check: only one exists
if(count($rows) > 1) {
    //the db should NOT be allowed to get this way!!!
    //...or, maybe I miswrote that JOIN query.
    error("More than one user with the email address $userAddy .  Exiting");
    exit_script();
}


// if none exist: ban posting.  send apologetic reject email
if(count($rows) == 0) {
    //XXX should add snippet of message they tried to post, or bounce back
    //the whole thing, or somethin...
    
    //Another potential issue: someone spoofs this address [unwittingly doing
    //the spoofing wrong, since it's not registered] -- then the recipient
    //of this bounce message will be very confused.  Need a "if you believe
    //you received this message in error..." type thing.

    mail($userAddy, "Posting failed",
"Sorry, but your posting to the Deme group ".$CURGROUP->getLongName()."
was rejected.  You posted from the email address $userAddy .
This address is not listed as the address of any member of the group.
If you do in fact belong to the group and wish to post from this email address,
you must first change your registered email address on the system.

Click here to change that: " .$CONF['DEME_URL_ROOT']."/meta/usersettings.php?mode=meta"
        );
    exit_script();
}

// Assume that user's identity
$username = $rows[0]['username'];


//Insert into DB: table msgs

$subj = $decode->getHeaderValue("Subject");
if( ! $subj ) {
    //assign a Re: [prev subject] default subject
    $subj = "[no subject]";
}
// strips out   Re: [groupname]   crap that may be at start
$subj = clean_subject($subj, $CURGROUP->getName());

//We need to decode the bodies before running getPlainTextBody()
$structure = $decode->decode(array('include_bodies' => true,
                                   'decode_bodies' => true,
                                   'decode_headers' => true));
// TODO: switch to a first pass with decode_bodies => false, because that's
// really expensive for big binary attachments...

$msgtext = mailinput_getPlainTextBody($structure);

//echo "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n";
//echo $msgtext;
//exit;


//echo "MSGTEXT:\n";
//echo ($msgtext);

echo $in_reply_to . "\n";

$msg_id = new_msg_in_db($db, $subj, $msgtext, $marea_id,
                        $in_reply_to, $username, 0);
error("Added msg_id=".$msg_id);

//Other considerations on incoming email
// What do we do with multiple other addys on the To: list?  Or the Cc: list?
// Let's store them in their own columns! thank goodness we have the new msg_id
// TODO implement this... the columns have already been added



//Mail out new message.
// Can NOT copy orig incoming message like a mailing list, of course.  Maybe
//  should copy parts instead of relying on standard components from a web
//  posting's notification mail?  e.g. the email's From: address instead of the
//  user's registered primary address, if we're allowing multiple email addys
//  per person.
//
// use MessageNotice class!
//
// To: gets the reply-to-msgXX where XX is THIS msg_id!
//   Not whatever the actual user To'd
// Will this will send THIS message to the Deme mail handler script again?
//  If so, need to add earlier checks to discount it.  Like X-Deme headers...
// Special X-Deme headers can't hurt.  Are a good check if we don't
//  trust the marea/group short names on the Cc: reply address.
//    X-Deme-Msg-Id
//    X-Deme-Group-Name
//    X-Deme-Marea-Id
//    X-Deme-Group-Url
// IMPLEMENTATION for all this is in Notices.php


$notice = new MessageNotice($username, $msg_id, $marea_id,
                            $subj, $msgtext);
$notice->notify();

echo "\n";
exit_script();

//------------ end of script actions -------------




//----------------- crap no longer used --------------------

//Test of php's builtin imap_rfc822_parse_adrlist()
//$to = "\"bla bla nice name\" <asfd-reply@groupname.groupspace.org>, brendano@groupspace.org";
//$a = imap_rfc822_parse_adrlist($to, "");
//print_r($a);
//exit_script();

// shifted this to looking at the argv[1] $recipient at the start of script

// $addyObjects = array_merge($addyObjs1, $addyObjs2);
// $postingAddyComponents = null;
// foreach($addyObjects as $addyObject) {
//     if( ! ($addyObject->host && $addyObject->mailbox)) continue;
//     $addy = $addyObject->mailbox ."@". $addyObject->host;
// //    echo $addy."\n";
//     $postingAddyComponents = parsePostingAddress($addy);
//     if($postingAddyComponents !== null) {
//         break;
//     }
// }

// if($postingAddyComponents === null) {
//     error_log("Didn't find a syntactically valid posting address");
//     exit;
// }

?>
